@@ -22,9 +22,12 @@ declare -A CHECKS=(
 	["mount-home"]="1"
 	["containerd-user"]="1"
 	["restart"]="1"
+	["snapshot-online"]="1"
+	["snapshot-offline"]="1"
 	["port-forwards"]="1"
 	["vmnet"]=""
 	["disk"]=""
+	["user-v2"]=""
 )
 
 case "$NAME" in
@@ -43,12 +46,20 @@ case "$NAME" in
 	# ● run-r2b459797f5b04262bfa79984077a65c7.service                                       loaded failed failed    /usr/bin/systemctl start man-db-cache-update
 	CHECKS["systemd-strict"]=
 	;;
+"9p")
+	CHECKS["snapshot-online"]=""
+	;;
 "vmnet")
 	CHECKS["vmnet"]=1
 	;;
 "test-misc")
 	CHECKS["disk"]=1
 	;;
+"net-user-v2")
+	CHECKS["port-forwards"]=""
+	CHECKS["snapshot-online"]=""
+	CHECKS["user-v2"]=1
+	;;
 esac
 
 if limactl ls -q | grep -q "$NAME"; then
@@ -303,6 +314,67 @@ if [[ -n ${CHECKS["restart"]} ]]; then
 	fi
 fi
 
+if [[ -n ${CHECKS["user-v2"]} ]]; then
+	INFO "Testing user-v2 network"
+	secondvm="$NAME-1"
+	limactl start "$FILE" --name "$secondvm" --tty=false
+	guestNewip="$(limactl shell "$secondvm" ip -4 -j addr show dev eth0 | jq -r '.[0].addr_info[0].local')"
+	INFO "IP of $secondvm is $guestNewip"
+	set -x
+	if ! limactl shell "$NAME" ping -c 1 "$guestNewip"; then
+		ERROR "Failed to do vm->vm communication via user-v2"
+		INFO "Stopping \"$secondvm\""
+		limactl stop "$secondvm"
+		INFO "Deleting \"$secondvm\""
+		limactl delete "$secondvm"
+		exit 1
+	fi
+	INFO "Stopping \"$secondvm\""
+	limactl stop "$secondvm"
+	INFO "Deleting \"$secondvm\""
+	limactl delete "$secondvm"
+	set +x
+fi
+if [[ -n ${CHECKS["snapshot-online"]} ]]; then
+	INFO "Testing online snapshots"
+	limactl shell "$NAME" sh -c 'echo foo > /tmp/test'
+	limactl snapshot create "$NAME" --tag snap1
+	got=$(limactl snapshot list "$NAME" --quiet)
+	expected="snap1"
+	INFO "snapshot list: expected=${expected} got=${got}"
+	if [ "$got" != "$expected" ]; then
+		ERROR "snapshot list did not return expected value"
+		exit 1
+	fi
+	limactl shell "$NAME" sh -c 'echo bar > /tmp/test'
+	limactl snapshot apply "$NAME" --tag snap1
+	got=$(limactl shell "$NAME" cat /tmp/test)
+	expected="foo"
+	INFO "snapshot apply: expected=${expected} got=${got}"
+	if [ "$got" != "$expected" ]; then
+		ERROR "snapshot apply did not restore snapshot"
+		exit 1
+	fi
+	limactl snapshot delete "$NAME" --tag snap1
+	limactl shell "$NAME" rm /tmp/test
+fi
+if [[ -n ${CHECKS["snapshot-offline"]} ]]; then
+	INFO "Testing offline snapshots"
+	limactl stop "$NAME"
+	sleep 3
+	limactl snapshot create "$NAME" --tag snap2
+	got=$(limactl snapshot list "$NAME" --quiet)
+	expected="snap2"
+	INFO "snapshot list: expected=${expected} got=${got}"
+	if [ "$got" != "$expected" ]; then
+		ERROR "snapshot list did not return expected value"
+		exit 1
+	fi
+	limactl snapshot apply "$NAME" --tag snap2
+	limactl snapshot delete "$NAME" --tag snap2
+	limactl start "$NAME"
+fi
+
 INFO "Stopping \"$NAME\""
 limactl stop "$NAME"
 sleep 3
