@@ -2,7 +2,6 @@ package guestagent
 
 import (
 	"context"
-	"encoding/binary"
 	"errors"
 	"reflect"
 	"sync"
@@ -12,15 +11,17 @@ import (
 	"github.com/elastic/go-libaudit/v2/auparse"
 	"github.com/lima-vm/lima/pkg/guestagent/api"
 	"github.com/lima-vm/lima/pkg/guestagent/iptables"
+	"github.com/lima-vm/lima/pkg/guestagent/kubernetesservice"
 	"github.com/lima-vm/lima/pkg/guestagent/procnettcp"
 	"github.com/lima-vm/lima/pkg/guestagent/timesync"
 	"github.com/sirupsen/logrus"
-	"github.com/yalue/native_endian"
+	"golang.org/x/sys/cpu"
 )
 
 func New(newTicker func() (<-chan time.Time, func()), iptablesIdle time.Duration) (Agent, error) {
 	a := &agent{
-		newTicker: newTicker,
+		newTicker:                newTicker,
+		kubernetesServiceWatcher: kubernetesservice.NewServiceWatcher(),
 	}
 
 	auditClient, err := libaudit.NewMulticastAuditClient(nil)
@@ -38,6 +39,7 @@ func New(newTicker func() (<-chan time.Time, func()), iptablesIdle time.Duration
 	}
 
 	go a.setWorthCheckingIPTablesRoutine(auditClient, iptablesIdle)
+	go a.kubernetesServiceWatcher.Start()
 	go a.fixSystemTimeSkew()
 	return a, nil
 }
@@ -48,10 +50,11 @@ type agent struct {
 	// reload /proc/net/tcp.
 	newTicker func() (<-chan time.Time, func())
 
-	worthCheckingIPTables   bool
-	worthCheckingIPTablesMu sync.RWMutex
-	latestIPTables          []iptables.Entry
-	latestIPTablesMu        sync.RWMutex
+	worthCheckingIPTables    bool
+	worthCheckingIPTablesMu  sync.RWMutex
+	latestIPTables           []iptables.Entry
+	latestIPTablesMu         sync.RWMutex
+	kubernetesServiceWatcher *kubernetesservice.ServiceWatcher
 }
 
 // setWorthCheckingIPTablesRoutine sets worthCheckingIPTables to be true
@@ -171,7 +174,7 @@ func (a *agent) Events(ctx context.Context, ch chan api.Event) {
 }
 
 func (a *agent) LocalPorts(ctx context.Context) ([]api.IPPort, error) {
-	if native_endian.NativeEndian() == binary.BigEndian {
+	if cpu.IsBigEndian {
 		return nil, errors.New("big endian architecture is unsupported, because I don't know how /proc/net/tcp looks like on big endian hosts")
 	}
 	var res []api.IPPort
@@ -232,6 +235,24 @@ func (a *agent) LocalPorts(ctx context.Context) ([]api.IPPort, error) {
 		}
 	}
 
+	kubernetesEntries := a.kubernetesServiceWatcher.GetPorts()
+	for _, entry := range kubernetesEntries {
+		found := false
+		for _, re := range res {
+			if re.Port == int(entry.Port) {
+				found = true
+			}
+		}
+
+		if !found {
+			res = append(res,
+				api.IPPort{
+					IP:   entry.IP,
+					Port: int(entry.Port),
+				})
+		}
+	}
+
 	return res, nil
 }
 
