@@ -48,6 +48,17 @@ func Validate(y LimaYAML, warn bool) error {
 		return fmt.Errorf("field `arch` must be %q, %q, or %q; got %q", X8664, AARCH64, RISCV64, *y.Arch)
 	}
 
+	switch *y.VMType {
+	case QEMU:
+		// NOP
+	case VZ:
+		if !IsNativeArch(*y.Arch) {
+			return fmt.Errorf("field `arch` must be %q for VZ; got %q", NewArch(runtime.GOARCH), *y.Arch)
+		}
+	default:
+		return fmt.Errorf("field `vmType` must be %q or %q; got %q", QEMU, VZ, *y.VMType)
+	}
+
 	if len(y.Images) == 0 {
 		return errors.New("field `images` must be set")
 	}
@@ -153,9 +164,14 @@ func Validate(y LimaYAML, warn bool) error {
 	for i, p := range y.Provision {
 		switch p.Mode {
 		case ProvisionModeSystem, ProvisionModeUser, ProvisionModeBoot:
+			if p.SkipDefaultDependencyResolution != nil {
+				return fmt.Errorf("field `provision[%d].mode` cannot set skipDefaultDependencyResolution, only valid on scripts of type %q",
+					i, ProvisionModeDependency)
+			}
+		case ProvisionModeDependency:
 		default:
-			return fmt.Errorf("field `provision[%d].mode` must be either %q, %q, or %q",
-				i, ProvisionModeSystem, ProvisionModeUser, ProvisionModeBoot)
+			return fmt.Errorf("field `provision[%d].mode` must one of %q, %q, %q, or %q",
+				i, ProvisionModeSystem, ProvisionModeUser, ProvisionModeBoot, ProvisionModeDependency)
 		}
 	}
 	needsContainerdArchives := (y.Containerd.User != nil && *y.Containerd.User) || (y.Containerd.System != nil && *y.Containerd.System)
@@ -281,7 +297,18 @@ func validateNetwork(y LimaYAML, warn bool) error {
 	for i, nw := range y.Networks {
 		field := fmt.Sprintf("networks[%d]", i)
 		if nw.Lima != "" {
-			if runtime.GOOS != "darwin" {
+			config, err := networks.Config()
+			if err != nil {
+				return err
+			}
+			if config.Check(nw.Lima) != nil {
+				return fmt.Errorf("field `%s.lima` references network %q which is not defined in networks.yaml", field, nw.Lima)
+			}
+			usernet, err := config.Usernet(nw.Lima)
+			if err != nil {
+				return err
+			}
+			if !usernet && runtime.GOOS != "darwin" {
 				return fmt.Errorf("field `%s.lima` is only supported on macOS right now", field)
 			}
 			if nw.Socket != "" {
@@ -296,13 +323,6 @@ func validateNetwork(y LimaYAML, warn bool) error {
 			if nw.SwitchPortDeprecated != 0 {
 				return fmt.Errorf("field `%s.switchPort` cannot be used with field `%s.lima`", field, field)
 			}
-			config, err := networks.Config()
-			if err != nil {
-				return err
-			}
-			if config.Check(nw.Lima) != nil {
-				return fmt.Errorf("field `%s.lima` references network %q which is not defined in networks.yaml", field, nw.Lima)
-			}
 		} else if nw.Socket != "" {
 			if nw.VZNAT != nil && *nw.VZNAT {
 				return fmt.Errorf("field `%s.socket` and field `%s.vzNAT` are mutually exclusive", field, field)
@@ -431,4 +451,7 @@ func warnExperimental(y LimaYAML) {
 	if y.Video.Display != nil && strings.Contains(*y.Video.Display, "vnc") {
 		logrus.Warn("`video.display: vnc` is experimental")
 	}
+	if y.Audio.Device != nil && *y.Audio.Device != "" {
+		logrus.Warn("`audio.device` is experimental")
+	}
 }
