@@ -9,11 +9,17 @@ import (
 	"os/exec"
 	"os/user"
 	"path/filepath"
+	"regexp"
 	"runtime"
 	"strconv"
 	"strings"
+	"time"
+
+	"github.com/lima-vm/lima/pkg/networks/usernet"
 
 	"github.com/coreos/go-semver/semver"
+	"github.com/digitalocean/go-qemu/qmp"
+	"github.com/digitalocean/go-qemu/qmp/raw"
 	"github.com/docker/go-units"
 	"github.com/lima-vm/lima/pkg/fileutils"
 	"github.com/lima-vm/lima/pkg/iso9660util"
@@ -34,6 +40,9 @@ type Config struct {
 	SSHLocalPort int
 }
 
+// MinimumQemuVersion is the minimum supported QEMU version
+const MinimumQemuVersion = "4.0.0"
+
 // EnsureDisk also ensures the kernel and the initrd
 func EnsureDisk(cfg Config) error {
 	diffDisk := filepath.Join(cfg.InstanceDir, filenames.DiffDisk)
@@ -87,13 +96,19 @@ func EnsureDisk(cfg Config) error {
 	if err != nil {
 		return err
 	}
+	baseDiskInfo, err := imgutil.GetInfo(baseDisk)
+	if err != nil {
+		return fmt.Errorf("failed to get the information of base disk %q: %w", baseDisk, err)
+	}
+	if err = imgutil.AcceptableAsBasedisk(baseDiskInfo); err != nil {
+		return fmt.Errorf("file %q is not acceptable as the base disk: %w", baseDisk, err)
+	}
+	if baseDiskInfo.Format == "" {
+		return fmt.Errorf("failed to inspect the format of %q", baseDisk)
+	}
 	args := []string{"create", "-f", "qcow2"}
 	if !isBaseDiskISO {
-		baseDiskFormat, err := imgutil.DetectFormat(baseDisk)
-		if err != nil {
-			return err
-		}
-		args = append(args, "-F", baseDiskFormat, "-b", baseDisk)
+		args = append(args, "-F", baseDiskInfo.Format, "-b", baseDisk)
 	}
 	args = append(args, diffDisk, strconv.Itoa(int(diskSize)))
 	cmd := exec.Command("qemu-img", args...)
@@ -118,6 +133,105 @@ func CreateDataDisk(dir, format string, size int) error {
 	return nil
 }
 
+func newQmpClient(cfg Config) (*qmp.SocketMonitor, error) {
+	qmpSock := filepath.Join(cfg.InstanceDir, filenames.QMPSock)
+	qmpClient, err := qmp.NewSocketMonitor("unix", qmpSock, 5*time.Second)
+	if err != nil {
+		return nil, err
+	}
+	return qmpClient, nil
+}
+
+func sendHmpCommand(cfg Config, cmd string, tag string) (string, error) {
+	qmpClient, err := newQmpClient(cfg)
+	if err != nil {
+		return "", err
+	}
+	if err := qmpClient.Connect(); err != nil {
+		return "", err
+	}
+	defer func() { _ = qmpClient.Disconnect() }()
+	rawClient := raw.NewMonitor(qmpClient)
+	logrus.Infof("Sending HMP %s command", cmd)
+	hmc := fmt.Sprintf("%s %s", cmd, tag)
+	return rawClient.HumanMonitorCommand(hmc, nil)
+}
+
+func execImgCommand(cfg Config, args ...string) (string, error) {
+	diffDisk := filepath.Join(cfg.InstanceDir, filenames.DiffDisk)
+	args = append(args, diffDisk)
+	logrus.Debugf("Running qemu-img %v command", args)
+	cmd := exec.Command("qemu-img", args...)
+	b, err := cmd.Output()
+	if err != nil {
+		return "", err
+	}
+	return string(b), err
+}
+
+func Del(cfg Config, run bool, tag string) error {
+	if run {
+		out, err := sendHmpCommand(cfg, "delvm", tag)
+		// there can still be output, even if no error!
+		if out != "" {
+			logrus.Warnf("output: %s", strings.TrimSpace(out))
+		}
+		return err
+	}
+	// -d  deletes a snapshot
+	_, err := execImgCommand(cfg, "snapshot", "-d", tag)
+	return err
+}
+
+func Save(cfg Config, run bool, tag string) error {
+	if run {
+		out, err := sendHmpCommand(cfg, "savevm", tag)
+		// there can still be output, even if no error!
+		if out != "" {
+			logrus.Warnf("output: %s", strings.TrimSpace(out))
+		}
+		return err
+	}
+	// -c  creates a snapshot
+	_, err := execImgCommand(cfg, "snapshot", "-c", tag)
+	return err
+}
+
+func Load(cfg Config, run bool, tag string) error {
+	if run {
+		out, err := sendHmpCommand(cfg, "loadvm", tag)
+		// there can still be output, even if no error!
+		if out != "" {
+			logrus.Warnf("output: %s", strings.TrimSpace(out))
+		}
+		return err
+	}
+	// -a  applies a snapshot
+	_, err := execImgCommand(cfg, "snapshot", "-a", tag)
+	return err
+}
+
+// List returns a space-separated list of all snapshots, with header and newlines
+func List(cfg Config, run bool) (string, error) {
+	if run {
+		out, err := sendHmpCommand(cfg, "info", "snapshots")
+		if err == nil {
+			out = strings.ReplaceAll(out, "\r", "")
+			out = strings.Replace(out, "List of snapshots present on all disks:\n", "", 1)
+			out = strings.Replace(out, "There is no snapshot available.\n", "", 1)
+		}
+		return out, err
+	}
+	// -l  lists all snapshots
+	args := []string{"snapshot", "-l"}
+	out, err := execImgCommand(cfg, args...)
+	if err == nil {
+		// remove the redundant heading, result is not machine-parseable
+		out = strings.Replace(out, "Snapshot list:\n", "", 1)
+	}
+	return out, err
+}
+
 func argValue(args []string, key string) (string, bool) {
 	if !strings.HasPrefix(key, "-") {
 		panic(fmt.Errorf("got unexpected key %q", key))
@@ -339,6 +453,16 @@ func Cmdline(cfg Config) (string, []string, error) {
 		return "", nil, err
 	}
 
+	version, err := getQemuVersion(exe)
+	if err != nil {
+		logrus.WithError(err).Warning("Failed to detect QEMU version")
+	} else {
+		logrus.Debugf("QEMU version %s detected", version.String())
+		if version.LessThan(*semver.New(MinimumQemuVersion)) {
+			logrus.Fatalf("QEMU %v is too old, %v or later required", version, MinimumQemuVersion)
+		}
+	}
+
 	// Architecture
 	accel := getAccel(*y.Arch)
 	if !strings.Contains(string(features.AccelHelp), accel) {
@@ -376,8 +500,9 @@ func Cmdline(cfg Config) (string, []string, error) {
 			// https://qemu-project.gitlab.io/qemu/system/invocation.html?highlight=tcg%20opts
 			// this will make sure each vCPU will be backed by 1 host user thread.
 			args = appendArgsIfNoConflict(args, "-accel", "tcg,thread=multi,tb-size=512")
-			// This will disable CPU S3 state.
+			// This will disable CPU S3/S4 state.
 			args = append(args, "-global", "ICH9-LPC.disable_s3=1")
+			args = append(args, "-global", "ICH9-LPC.disable_s4=1")
 		} else if runtime.GOOS == "windows" && accel == "whpx" {
 			// whpx: injection failed, MSI (0, 0) delivery: 0, dest_mode: 0, trigger mode: 0, vector: 0
 			args = appendArgsIfNoConflict(args, "-machine", "q35,accel="+accel+",kernel-irqchip=off")
@@ -451,14 +576,24 @@ func Cmdline(cfg Config) (string, []string, error) {
 	}
 	if isBaseDiskCDROM {
 		args = appendArgsIfNoConflict(args, "-boot", "order=d,splash-time=0,menu=on")
-		args = append(args, "-drive", fmt.Sprintf("file=%s,media=cdrom,readonly=on", baseDisk))
+		args = append(args, "-drive", fmt.Sprintf("file=%s,format=raw,media=cdrom,readonly=on", baseDisk))
 	} else {
 		args = appendArgsIfNoConflict(args, "-boot", "order=c,splash-time=0,menu=on")
 	}
 	if diskSize, _ := units.RAMInBytes(*cfg.LimaYAML.Disk); diskSize > 0 {
 		args = append(args, "-drive", fmt.Sprintf("file=%s,if=virtio,discard=on", diffDisk))
 	} else if !isBaseDiskCDROM {
-		args = append(args, "-drive", fmt.Sprintf("file=%s,if=virtio,discard=on", baseDisk))
+		baseDiskInfo, err := imgutil.GetInfo(baseDisk)
+		if err != nil {
+			return "", nil, fmt.Errorf("failed to get the information of %q: %w", baseDisk, err)
+		}
+		if err = imgutil.AcceptableAsBasedisk(baseDiskInfo); err != nil {
+			return "", nil, fmt.Errorf("file %q is not acceptable as the base disk: %w", baseDisk, err)
+		}
+		if baseDiskInfo.Format == "" {
+			return "", nil, fmt.Errorf("failed to inspect the format of %q", baseDisk)
+		}
+		args = append(args, "-drive", fmt.Sprintf("file=%s,format=%s,if=virtio,discard=on", baseDisk, baseDiskInfo.Format))
 	}
 	for _, extraDisk := range extraDisks {
 		args = append(args, "-drive", fmt.Sprintf("file=%s,if=virtio,discard=on", extraDisk))
@@ -485,9 +620,20 @@ func Cmdline(cfg Config) (string, []string, error) {
 	}
 
 	// Network
-	args = append(args, "-netdev", fmt.Sprintf("user,id=net0,net=%s,dhcpstart=%s,hostfwd=tcp:127.0.0.1:%d-:22",
-		networks.SlirpNetwork, networks.SlirpIPAddress, cfg.SSHLocalPort))
+	//Configure default usernetwork with limayaml.MACAddress(driver.Instance.Dir) for eth0 interface
+	firstUsernetIndex := limayaml.FirstUsernetIndex(y)
+	if firstUsernetIndex == -1 {
+		args = append(args, "-netdev", fmt.Sprintf("user,id=net0,net=%s,dhcpstart=%s,hostfwd=tcp:127.0.0.1:%d-:22",
+			networks.SlirpNetwork, networks.SlirpIPAddress, cfg.SSHLocalPort))
+	} else {
+		qemuSock, err := usernet.Sock(y.Networks[firstUsernetIndex].Lima, usernet.QEMUSock)
+		if err != nil {
+			return "", nil, err
+		}
+		args = append(args, "-netdev", fmt.Sprintf("socket,id=net0,fd={{ fd_connect %q }}", qemuSock))
+	}
 	args = append(args, "-device", "virtio-net-pci,netdev=net0,mac="+limayaml.MACAddress(cfg.InstanceDir))
+
 	for i, nw := range y.Networks {
 		var vdeSock string
 		if nw.Lima != "" {
@@ -495,29 +641,50 @@ func Cmdline(cfg Config) (string, []string, error) {
 			if err != nil {
 				return "", nil, err
 			}
-			socketVMNetOk, err := nwCfg.IsDaemonInstalled(networks.SocketVMNet)
+
+			//Handle usernet connections
+			isUsernet, err := nwCfg.Usernet(nw.Lima)
 			if err != nil {
 				return "", nil, err
 			}
-			if socketVMNetOk {
-				logrus.Debugf("Using socketVMNet (%q)", nwCfg.Paths.SocketVMNet)
-				if vdeVMNetOk, _ := nwCfg.IsDaemonInstalled(networks.VDEVMNet); vdeVMNetOk {
-					logrus.Debugf("Ignoring vdeVMNet (%q), as socketVMNet (%q) is available and has higher precedence", nwCfg.Paths.VDEVMNet, nwCfg.Paths.SocketVMNet)
+			if isUsernet {
+				if i == firstUsernetIndex {
+					continue
 				}
-				sock, err := networks.Sock(nw.Lima)
+				qemuSock, err := usernet.Sock(nw.Lima, usernet.QEMUSock)
 				if err != nil {
 					return "", nil, err
 				}
-				args = append(args, "-netdev", fmt.Sprintf("socket,id=net%d,fd={{ fd_connect %q }}", i+1, sock))
-			} else if nwCfg.Paths.VDEVMNet != "" {
-				logrus.Warn("vdeVMNet is deprecated, use socketVMNet instead (See docs/network.md)")
-				vdeSock, err = networks.VDESock(nw.Lima)
+				args = append(args, "-netdev", fmt.Sprintf("socket,id=net%d,fd={{ fd_connect %q }}", i+1, qemuSock))
+				args = append(args, "-device", fmt.Sprintf("virtio-net-pci,netdev=net%d,mac=%s", i+1, nw.MACAddress))
+			} else {
+				if runtime.GOOS != "darwin" {
+					return "", nil, fmt.Errorf("networks.yaml '%s' configuration is only supported on macOS right now", nw.Lima)
+				}
+				socketVMNetOk, err := nwCfg.IsDaemonInstalled(networks.SocketVMNet)
 				if err != nil {
 					return "", nil, err
 				}
+				if socketVMNetOk {
+					logrus.Debugf("Using socketVMNet (%q)", nwCfg.Paths.SocketVMNet)
+					if vdeVMNetOk, _ := nwCfg.IsDaemonInstalled(networks.VDEVMNet); vdeVMNetOk {
+						logrus.Debugf("Ignoring vdeVMNet (%q), as socketVMNet (%q) is available and has higher precedence", nwCfg.Paths.VDEVMNet, nwCfg.Paths.SocketVMNet)
+					}
+					sock, err := networks.Sock(nw.Lima)
+					if err != nil {
+						return "", nil, err
+					}
+					args = append(args, "-netdev", fmt.Sprintf("socket,id=net%d,fd={{ fd_connect %q }}", i+1, sock))
+				} else if nwCfg.Paths.VDEVMNet != "" {
+					logrus.Warn("vdeVMNet is deprecated, use socketVMNet instead (See docs/network.md)")
+					vdeSock, err = networks.VDESock(nw.Lima)
+					if err != nil {
+						return "", nil, err
+					}
+				}
+				// TODO: should we also validate that the socket exists, or do we rely on the
+				// networks reconciler to throw an error when the network cannot start?
 			}
-			// TODO: should we also validate that the socket exists, or do we rely on the
-			// networks reconciler to throw an error when the network cannot start?
 		} else if nw.Socket != "" {
 			args = append(args, "-netdev", fmt.Sprintf("socket,id=net%d,fd={{ fd_connect %q }}", i+1, nw.Socket))
 		} else if nw.VNLDeprecated != "" {
@@ -555,6 +722,18 @@ func Cmdline(cfg Config) (string, []string, error) {
 	// virtio-rng-pci accelerates starting up the OS, according to https://wiki.gentoo.org/wiki/QEMU/Options
 	args = append(args, "-device", "virtio-rng-pci")
 
+	// Sound
+	if *y.Audio.Device != "" {
+		id := "default"
+		// audio device
+		audiodev := *y.Audio.Device
+		audiodev += fmt.Sprintf(",id=%s", id)
+		args = append(args, "-audiodev", audiodev)
+		// audio controller
+		args = append(args, "-device", "ich9-intel-hda")
+		// audio codec
+		args = append(args, "-device", fmt.Sprintf("hda-output,audiodev=%s", id))
+	}
 	// Graphics
 	if *y.Video.Display != "" {
 		display := *y.Video.Display
@@ -670,6 +849,31 @@ func getAccel(arch limayaml.Arch) string {
 	return "tcg"
 }
 
+func parseQemuVersion(output string) (*semver.Version, error) {
+	lines := strings.Split(output, "\n")
+	regex := regexp.MustCompile(`^QEMU emulator version (\d+\.\d+\.\d+)`)
+	matches := regex.FindStringSubmatch(lines[0])
+	if len(matches) == 2 {
+		return semver.New(matches[1]), nil
+	}
+	return &semver.Version{}, fmt.Errorf("failed to parse %v", output)
+}
+
+func getQemuVersion(qemuExe string) (*semver.Version, error) {
+	var (
+		stdout bytes.Buffer
+		stderr bytes.Buffer
+	)
+	cmd := exec.Command(qemuExe, "--version")
+	cmd.Stdout = &stdout
+	cmd.Stderr = &stderr
+	if err := cmd.Run(); err != nil {
+		return nil, fmt.Errorf("failed to run %v: stdout=%q, stderr=%q", cmd.Args, stdout.String(), stderr.String())
+	}
+
+	return parseQemuVersion(stdout.String())
+}
+
 func getFirmware(qemuExe string, arch limayaml.Arch) (string, error) {
 	switch arch {
 	case limayaml.X8664, limayaml.AARCH64:
