@@ -20,6 +20,8 @@ import (
 	"github.com/digitalocean/go-qemu/qmp/raw"
 	"github.com/lima-vm/lima/pkg/driver"
 	"github.com/lima-vm/lima/pkg/limayaml"
+	"github.com/lima-vm/lima/pkg/networks/usernet"
+	"github.com/lima-vm/lima/pkg/store"
 	"github.com/lima-vm/lima/pkg/store/filenames"
 	"github.com/sirupsen/logrus"
 )
@@ -100,6 +102,12 @@ func (l *LimaQemuDriver) Start(ctx context.Context) (chan error, error) {
 	go func() {
 		l.qWaitCh <- qCmd.Wait()
 	}()
+	go func() {
+		if usernetIndex := limayaml.FirstUsernetIndex(l.Yaml); usernetIndex != -1 {
+			client := newUsernetClient(l.Yaml.Networks[usernetIndex].Lima)
+			err = client.ResolveAndForwardSSH(limayaml.MACAddress(l.Instance.Dir), l.SSHLocalPort)
+		}
+	}()
 	return l.qWaitCh, nil
 }
 
@@ -189,6 +197,13 @@ func (l *LimaQemuDriver) removeVNCFiles() error {
 
 func (l *LimaQemuDriver) shutdownQEMU(ctx context.Context, timeout time.Duration, qCmd *exec.Cmd, qWaitCh <-chan error) error {
 	logrus.Info("Shutting down QEMU with ACPI")
+	if usernetIndex := limayaml.FirstUsernetIndex(l.Yaml); usernetIndex != -1 {
+		client := newUsernetClient(l.Yaml.Networks[usernetIndex].Lima)
+		err := client.UnExposeSSH(l.SSHLocalPort)
+		if err != nil {
+			logrus.Warnf("Failed to remove SSH binding for port %d", l.SSHLocalPort)
+		}
+	}
 	qmpSockPath := filepath.Join(l.Instance.Dir, filenames.QMPSock)
 	qmpClient, err := qmp.NewSocketMonitor("unix", qmpSockPath, 5*time.Second)
 	if err != nil {
@@ -235,6 +250,14 @@ func (l *LimaQemuDriver) killQEMU(_ context.Context, _ time.Duration, qCmd *exec
 	return qWaitErr
 }
 
+func newUsernetClient(nwName string) *usernet.Client {
+	endpointSock, err := usernet.Sock(nwName, usernet.EndpointSock)
+	if err != nil {
+		return nil
+	}
+	return usernet.NewClient(endpointSock)
+}
+
 func logPipeRoutine(r io.Reader, header string) {
 	scanner := bufio.NewScanner(r)
 	for scanner.Scan() {
@@ -243,6 +266,42 @@ func logPipeRoutine(r io.Reader, header string) {
 	}
 }
 
+func (l *LimaQemuDriver) DeleteSnapshot(ctx context.Context, tag string) error {
+	qCfg := Config{
+		Name:        l.Instance.Name,
+		InstanceDir: l.Instance.Dir,
+		LimaYAML:    l.Yaml,
+	}
+	return Del(qCfg, l.Instance.Status == store.StatusRunning, tag)
+}
+
+func (l *LimaQemuDriver) CreateSnapshot(ctx context.Context, tag string) error {
+	qCfg := Config{
+		Name:        l.Instance.Name,
+		InstanceDir: l.Instance.Dir,
+		LimaYAML:    l.Yaml,
+	}
+	return Save(qCfg, l.Instance.Status == store.StatusRunning, tag)
+}
+
+func (l *LimaQemuDriver) ApplySnapshot(ctx context.Context, tag string) error {
+	qCfg := Config{
+		Name:        l.Instance.Name,
+		InstanceDir: l.Instance.Dir,
+		LimaYAML:    l.Yaml,
+	}
+	return Load(qCfg, l.Instance.Status == store.StatusRunning, tag)
+}
+
+func (l *LimaQemuDriver) ListSnapshots(ctx context.Context) (string, error) {
+	qCfg := Config{
+		Name:        l.Instance.Name,
+		InstanceDir: l.Instance.Dir,
+		LimaYAML:    l.Yaml,
+	}
+	return List(qCfg, l.Instance.Status == store.StatusRunning)
+}
+
 type qArgTemplateApplier struct {
 	files []*os.File
 }
