@@ -37,6 +37,9 @@ const (
 	SerialLog          = "serial.log"
 	SerialSock         = "serial.sock"
 	SSHSock            = "ssh.sock"
+	SSHConfig          = "ssh.config"
+	VNCDisplayFile     = "vncdisplay"
+	VNCPasswordFile    = "vncpassword"
 	GuestAgentSock     = "ga.sock"
 	HostAgentPID       = "ha.pid"
 	HostAgentSock      = "ha.sock"
