@@ -36,6 +36,11 @@ func New(driver *driver.BaseDriver) *LimaVzDriver {
 }
 
 func (l *LimaVzDriver) Validate() error {
+	//Calling NewEFIBootLoader to do required version check for latest APIs
+	_, err := vz.NewEFIBootLoader()
+	if errors.Is(err, vz.ErrUnsupportedOSVersion) {
+		return fmt.Errorf("VZ driver requires macOS 13 or higher to run")
+	}
 	if *l.Yaml.MountType == limayaml.NINEP {
 		return fmt.Errorf("field `mountType` must be %q or %q for VZ driver , got %q", limayaml.REVSSHFS, limayaml.VIRTIOFS, *l.Yaml.MountType)
 	}
@@ -46,6 +51,7 @@ func (l *LimaVzDriver) Validate() error {
 		"Arch",
 		"Images",
 		"CPUs",
+		"CPUType",
 		"Memory",
 		"Disk",
 		"Mounts",
@@ -65,10 +71,21 @@ func (l *LimaVzDriver) Validate() error {
 		"CACertificates",
 		"Rosetta",
 		"AdditionalDisks",
+		"Audio",
 	); len(unknown) > 0 {
 		logrus.Warnf("Ignoring: vmType %s: %+v", *l.Yaml.VMType, unknown)
 	}
 
+	if !limayaml.IsNativeArch(*l.Yaml.Arch) {
+		return fmt.Errorf("unsupported arch: %q", *l.Yaml.Arch)
+	}
+
+	for k, v := range l.Yaml.CPUType {
+		if v != "" {
+			logrus.Warnf("Ignoring: vmType %s: cpuType[%q]: %q", *l.Yaml.VMType, k, v)
+		}
+	}
+
 	for i, image := range l.Yaml.Images {
 		if unknown := reflectutil.UnknownNonEmptyFields(image, "File"); len(unknown) > 0 {
 			logrus.Warnf("Ignoring: vmType %s: images[%d]: %+v", *l.Yaml.VMType, i, unknown)
@@ -96,6 +113,11 @@ func (l *LimaVzDriver) Validate() error {
 			logrus.Warnf("Ignoring: vmType %s: networks[%d]: %+v", *l.Yaml.VMType, i, unknown)
 		}
 	}
+
+	audioDevice := *l.Yaml.Audio.Device
+	if audioDevice != "" && audioDevice != "vz" {
+		logrus.Warnf("field `audio.device` must be %q for VZ driver , got %q", "vz", audioDevice)
+	}
 	return nil
 }
 
